@@ -50,61 +50,93 @@ def get_default_repo_address():
     #     host=cfg.get('default_host',host)
     #     repo=cfg.get('default_repo',repo)
     return '%s/%s/%s'%(host,username,repo)
+class Address:
+    def __init__(self,protocol,host,username,repo_name,branch_name,rel_path):
+        self.protocol=protocol
+        self.host=host
+        self.username=username
+        self.repo_name=repo_name
+        self.branch_name=branch_name
+        self.rel_path=rel_path
+    def to_url(self):
+        return '%s://%s/%s/%s:'%(self.protocol,self.host,self.username,self.repo_name)+join_path(self.branch_name,self.rel_path)
+    @classmethod
+    def from_url(cls,url):
+
+        protocol,repo_address,branch_name,rel_path=cls.parse_address(url)
+        host,username,repo_name=repo_address.split('/')
+        return cls(protocol,host,username,repo_name,branch_name,rel_path)
+    @classmethod
+    def parse_address(cls,address):
+        '''
+           address: [[[{host}/]{username}/]{repo_name}:]{branch_name}[/{relative_path}]
+           '''
+
+        def parse_rest(rest: str):
+            rest = rest.strip('/')
+            if '/' in rest:
+                branch_name, relative_path = rest.split('/', maxsplit=1)
+            else:
+                branch_name = rest
+                relative_path = '/'
+            return branch_name, relative_path
+
+        repo_address, rest = address.rsplit(':', maxsplit=1)
+        protocol = "freehub"
+        if "://" in repo_address:
+            protocol, repo_address = repo_address.split('://', maxsplit=1)
+        branch_name, relative_path = parse_rest(rest)
+        return protocol, repo_address, branch_name, relative_path
+    @classmethod
+    def get_complete_address(cls,address:str):
+        address = address.replace('\\', '/')
+        protocol="freehub"
+        if "://" in address:
+            protocol,address=address.split('://',maxsplit=1)
+        if ':' in address:
+            assert address.count(':') == 1
+            repo_address, rest = address.split(':')
+            parts = repo_address.split('/')
+            assert len(parts) >= 1
+            assert len(parts) <= 3
+            default_parts = get_default_repo_address().split('/')
+            default_parts[-1] = parts[-1]
+            if len(parts) >= 2:
+                default_parts[-2] = parts[-2]
+            if len(parts) >= 3:
+                default_parts[-3] = parts[-3]
+            repo_address = '/'.join(default_parts)
+        else:
+            repo_address = get_default_repo_address()
+            rest = address
+        addr = protocol+"://"+':'.join([repo_address, rest])
+        return addr
+    def get_repo_url(self):
+        host, username_, repo = self.host,self.username,self.repo_name
+        cfg = _read_config()
+
+        if cfg and 'password' in cfg and 'username' in cfg:
+            username = cfg.get('username')
+            password = cfg.get('password')
+            repo_url = 'https://%s:%s@%s/%s/%s' % (username, password, host, username_, repo)
+            # print("repo_url:", 'https://%s:******@%s/%s/%s' % (username, host, username_, repo))
+        else:
+            # username=DEFAULT_USERNAME
+            # password=DEFAULT_PASSWORD
+            repo_url = 'https://%s/%s/%s' % (host, username_, repo)
+            # print("repo_url:",repo_url)
+        return repo_url
+
+
 
 def get_complete_address(address):
-    address=address.replace('\\','/')
-    if ':' in address:
-        assert address.count(':') == 1
-        repo_address, rest = address.split(':')
-        parts=repo_address.split('/')
-        assert len(parts)>=1
-        assert len(parts)<=3
-        default_parts=get_default_repo_address().split('/')
-        default_parts[-1]=parts[-1]
-        if len(parts)>=2:
-            default_parts[-2]=parts[-2]
-        if len(parts)>=3:
-            default_parts[-3]=parts[-3]
-        repo_address='/'.join(default_parts)
-    else:
-        repo_address = get_default_repo_address()
-        rest = address
-    addr= ':'.join([repo_address,rest])
-    # print('Complete address:',addr)
-    return addr
+    return Address.get_complete_address(address)
 def parse_address(address):
     '''
        address: [[[{host}/]{username}/]{repo_name}:]{branch_name}[/{relative_path}]
        '''
-    def parse_rest(rest:str):
-        rest=rest.strip('/')
-        if '/' in rest:
-            branch_name,relative_path=rest.split('/',maxsplit=1)
-        else:
-            branch_name=rest
-            relative_path='/'
-        return branch_name,relative_path
+    return Address.parse_address(address)
 
-    assert address.count(':') == 1
-    repo_address, rest = address.split(':')
-    branch_name, relative_path = parse_rest(rest)
-    return repo_address,branch_name,relative_path
-def get_repo_url(repo_address:str):
-    host,username_,repo=repo_address.split('/')
-
-    cfg=_read_config()
-
-    if cfg and 'password' in cfg and 'username' in cfg:
-        username=cfg.get('username')
-        password=cfg.get('password')
-        repo_url = 'https://%s:%s@%s/%s/%s' % (username, password, host, username_, repo)
-        # print("repo_url:", 'https://%s:******@%s/%s/%s' % (username, host, username_, repo))
-    else:
-        # username=DEFAULT_USERNAME
-        # password=DEFAULT_PASSWORD
-        repo_url = 'https://%s/%s/%s' % (host, username_, repo)
-        # print("repo_url:",repo_url)
-    return repo_url
 
 
 
@@ -171,36 +203,31 @@ def freehub_download(address,dst_path,overwrite=False,**kwargs):
     {cache_dir}/{relative_path}
     relative_path: can be a path pattern, e.g. *.txt
     '''
+    a=Address.from_url(address)
 
-    repo_address, branch_name,relative_path=parse_address(address)
-    repo_url=get_repo_url(repo_address)
-    cache_dir=download_branch_to_cache(repo_url,branch_name,overwrite=overwrite,**kwargs)
-    copy_repo_files_to(cache_dir,relative_path,dst_path,branch_name)
 
+    cache_dir=download_branch_to_cache(a.get_repo_url(),a.branch_name,overwrite=overwrite,**kwargs)
+    copy_repo_files_to(cache_dir,a.rel_path,dst_path,a.branch_name)
 
 
 def freehub_upload(src_path,address,overwrite=False,**kwargs):
-    repo_address, branch_name,relative_path=parse_address(address)
-    repo_url = get_repo_url(repo_address)
-    upload_to_remote(repo_url,branch_name,relative_path,src_path,overwrite=overwrite,**kwargs)
+    addr=Address.from_url(address)
+    upload_to_remote(addr.get_repo_url(),addr.branch_name,addr.rel_path,src_path,overwrite=overwrite,**kwargs)
 def fetch(address,**kwargs):
-    repo_address, branch_name, relative_path = parse_address(address)
-    repo_url = get_repo_url(repo_address)
-    cache_dir = download_branch_to_cache(repo_url, branch_name, overwrite=True,**kwargs)
-    path = join_path(cache_dir, relative_path)
+    addr = Address.from_url(address)
+    cache_dir = download_branch_to_cache(addr.get_repo_url(), addr.branch_name, overwrite=True,**kwargs)
+    path = join_path(cache_dir, addr.rel_path)
     return path
 def freehub_run(address,**kwargs):
     ''''''
-    repo_address, branch_name, relative_path = parse_address(address)
-    repo_url = get_repo_url(repo_address)
-    cache_dir = download_branch_to_cache(repo_url, branch_name, overwrite=True,**kwargs)
-    script_path=join_path(cache_dir,relative_path)
+    addr = Address.from_url(address)
+    cache_dir = download_branch_to_cache(addr.get_repo_url(), addr.branch_name, overwrite=True,**kwargs)
+    script_path=join_path(cache_dir,addr.rel_path)
     os.system(script_path)
 def freehub_cat(address:str,**kwargs):
-    repo_address, branch_name, relative_path = parse_address(address)
-    repo_url = get_repo_url(repo_address)
-    cache_dir = download_branch_to_cache(repo_url, branch_name, overwrite=True,**kwargs)
-    file_path = join_path(cache_dir, relative_path)
+    addr = Address.from_url(address)
+    cache_dir = download_branch_to_cache(addr.get_repo_url(), addr.branch_name, overwrite=True,**kwargs)
+    file_path = join_path(cache_dir, addr.rel_path)
     with open(file_path,'r') as f:
         print(f.read())
 def freehub_update():
@@ -225,23 +252,21 @@ def freehub_search(pattern: str,**kwargs):
     :return:
     '''
     # convert repo_address into repo_url
-    repo_address, branch_name_pattern, relative_path_pattern = parse_address(pattern)
-    repo_url = get_repo_url(repo_address)
-    assert relative_path_pattern != '/'
-    cache_dir = download_branch_to_cache(repo_url, branch_name_pattern, overwrite=True,**kwargs)
-    fs = glob.glob(join_path(cache_dir, relative_path_pattern), recursive=True)
+    addr = Address.from_url(pattern)
+    assert addr.rel_path != '/'
+    cache_dir = download_branch_to_cache(addr.get_repo_url(), addr.branch_name, overwrite=True,**kwargs)
+    fs = glob.glob(join_path(cache_dir, addr.rel_path), recursive=True)
     for f in fs:
         rel_path = get_relative_path(cache_dir, f)
         print(rel_path)
 
 def freehub_branch_search(pattern: str):
-    repo_address, branch_name_pattern, relative_path_pattern = parse_address(pattern)
-    repo_url = get_repo_url(repo_address)
-    assert relative_path_pattern=='/'
-    branches = git_ops.list_remote_branch(repo_url)
+    addr=Address.from_url(pattern)
+    assert addr.rel_path=='/'
+    branches = git_ops.list_remote_branch(addr.get_repo_url())
     for b in branches:
-        if branch_name_pattern:
-            if fnmatch.fnmatch(b, branch_name_pattern):
+        if addr.branch_name:
+            if fnmatch.fnmatch(b, addr.branch_name):
                 print(b)
         else:
             print(b)
