@@ -27,6 +27,12 @@ class Cli:
     def fetch(cls,address:str):
         raise NotImplementedError(cls.fetch)
     @classmethod
+    def pull(cls, address: str):
+        raise NotImplementedError(cls.pull)
+    @classmethod
+    def sync(cls, address: str):
+        raise NotImplementedError(cls.sync)
+    @classmethod
     def publish(cls,address,str):
         raise NotImplementedError(cls.publish)
     @classmethod
